#ifndef CONSOLHAWF3MAT_H
#define CONSOLHAWF3MAT_H

#include "genfile.h"

class con_hawf3mat
{
 public:
  con_hawf3mat();    //constructor
  ~con_hawf3mat();   //destructor

  void read(XFILE *in);
  void print(FILE *out);
  double get_betas(long ipp);
  double get_cps(double t);
  double get_rhos(double t);
  double get_cdiff(double pw,double pg,double t);
  double get_dg(double pc,double pg,double t,long ipp);
  double get_sw(double pw, double pg, double t, long ipp);
  double get_ds_dpc(double pw, double pg, double t, long ipp);
  double get_se(double sw);
  double get_ds_dt(double pw, double pg, double t, long ipp);
  double get_kintr(double pw, double pg, double t, long ipp);
  double get_krw(double pw, double pg, double t, long ipp);
  double get_krg(double pw, double pg, double t, long ipp);

  double get_alpha(double pw, double pg, double t,long ipp);
  double get_porosity(double pw, double pg,double t,long ipp);
  double get_ks(double pw,double pg,double t,long ipp);
  double get_kt(double pw,double pg,double t,long ipp);

  double get_rhogw(double pw,double pg,double t);
  double get_cpgw();
  double get_pgw(double pw,double pg,double t);
  double get_dpgw_dpc(double pw,double pg,double t);
  double get_dpgw_dt(double pw,double pg,double t);
  double get_pgws(double t);
  double get_dpgws_dt(double t);
  double get_rhow(double t);
  double get_kw(double pw,double pg,double t,long ipp);
  double get_betasg(double pw,double pg,double t,long ipp);
  double get_betasw(double pw,double pg,double t,long ipp);
  double get_betasgw(double pw,double pg,double t,long ipp);
  double get_drhow_dt(double t);
  double get_dhvap(double t);
  double get_muw(double t);
  double get_cpw();
  double get_lambdaw(double t);
  double get_betaw(double t);
  double get_mg(double pw,double pg,double t);
  double get_rhog(double pw,double pg,double t);
  double get_mug(double pw,double pg,double t);
  double get_muga(double t);
  double get_mugw(double t);
  double get_rhoga(double pw,double pg,double t);
  double get_cpga();
  double get_rhocp(double pw,double pg,double t,long ipp);
  double get_cpg(double pw,double pg,double t);
  double get_lambdaeff(double pw,double pg,double t,long ipp);

  void matcond (matrix &d,long ri,long ci,long ipp);
  void matcap (double &c,long ri,long ci,long ipp);
  
  void matcond1d (matrix &d,long ri,long ci,long ipp);
  void matcond2d (matrix &d,long ri,long ci,long ipp);
  void matcond3d (matrix &d,long ri,long ci,long ipp);

  void matcond2 (matrix &d,long ri,long ci,long ipp);
  
  void matcond1d_2 (matrix &d,long ri,long ci,long ipp);
  void matcond2d_2 (matrix &d,long ri,long ci,long ipp);
  void matcond3d_2 (matrix &d,long ri,long ci,long ipp);

  void rhs_volume (matrix &d,long ri,long ci,long ipp);
  void rhs1d1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d1 (matrix &d,long ri,long ci,long ipp);
  void rhs3d1 (matrix &d,long ri,long ci,long ipp);

  double transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp);
  double transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp,int flag);
  double transmission_nodval(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp);
  double transmission_flux(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp);

  void gaspress_check(double pw,double &pg,double t,long ipp);
  void waterpress_check(double &pw,double pg,double t,long ipp);

  void values_correction (vector &nv);

  double get_kww(double pw,double pg,double t,long ipp);
  double get_capww(double pw,double pg,double t,long ipp);
  double get_kwg(double pw,double pg,double t,long ipp);
  double get_capwg(double pw,double pg,double t,long ipp);
  double get_kwt(double pw,double pg,double t,long ipp);
  double get_capwt(double pw,double pg,double t,long ipp);

  double get_kgg(double pw,double pg,double t,long ipp);
  double get_capgg(double pw,double pg,double t,long ipp);
  double get_kgw(double pw,double pg,double t,long ipp);
  double get_capgw(double pw,double pg,double t,long ipp);
  double get_kgt(double pw,double pg,double t,long ipp);
  double get_capgt(double pw,double pg,double t,long ipp);

  double get_ktt1(double pw,double pg,double t,long ipp);
  double get_ktt2(double pw,double pg,double t,long ipp);
  double get_captt(double pw,double pg,double t,long ipp);
  double get_ktg(double pw,double pg,double t,long ipp);
  double get_captg(double pw,double pg,double t,long ipp);
  double get_ktw(double pw,double pg,double t,long ipp);
  double get_captw(double pw,double pg,double t,long ipp);

  double get_ktt2a(double pw,double pg,double t,long ipp);
  double get_ktt2b(double pw,double pg,double t,long ipp);
  double get_ktt2c(double pw,double pg,double t,long ipp);
  double get_ktt2d(double pw,double pg,double t,long ipp);

  double get_fw1(double pw,double pg,double t,long ipp);
  double get_fg(double pw,double pg,double t,long ipp);
  double get_ft1(double pw,double pg,double t,long ipp);

  double get_transmission_transcoeff_ww(double pw,double pg,double t,long bc,long ipp);
  double get_transmission_transcoeff_ww(double pw,double pg,double t,long bc,long ipp,int flag);
  double get_transmission_nodval_ww(double bv,double pw,double pg,double t,long bc,long ipp);
  double get_transmission_flux_ww(double bv,double pw,double pg,double t,long bc,long ipp);

  double get_transmission_transcoeff_tt(double pw,double pg,double t,long bc,long ipp);
  double get_transmission_nodval_tt(double bv,double trr,double pw,double pg,double t,long bc,long ipp);
  double get_transmission_flux_tt(double bv,double trr,double pw,double pg,double t,long bc,long ipp);

  double get_othervalue(long compother,long ipp,double *r);
  void print_othervalue_name(FILE *out,long compother);
  void updateval (long ipp);
  void initval(long ipp);
  
  /// returns ordered dof names
  void give_dof_names(namevart *dofname, long ntm);

  double give_water_pressure(long ipp);
  double give_gas_pressure(long ipp);
  double give_pore_pressure(long ipp);
  double give_suction(long ipp);
  double give_saturation_degree(long ipp);

  /// marks required non-transport quantities
  void give_reqntq(long *antq);

 private: 
  
  heatairwaterflowtype model_type;
  double ma;//molar mass of dry air
  double mw;//molar mass of water
  double gasr;//universal gas constant

  double t0;//[K] reference temperature 
  double p0;//Pa reference athmospheric pressure

  // PHYSICAL PROPERTIES OF DRY AIR
  double muga0,alphaa,betaa;

  // PHYSICAL PROPERTIES OF WATER VAPOUR
  // from D.Gawin, F.Pesavento (PRVAP.f90)
  double dv0,bv;
  double c8,c9,c10,c11,c12,c13;
  double mugw0,alphaw;

  // PHYSICAL PROPERTIES OF WATER
  // from Dariusz Gawin (WATPROP.f90)
  double rhow0;//density at refernce temperature and pressure
  double tcr;//[K] critical temperature of water
  double cwat;
  double betawat;
  double hvap0;//latent heat of vaporization at reference temperature
  double a0,a1,a2;
  double a3,a4,a5;
  double b0,b1,b2;
  double b3,b4,b5;
  double pr1,prif;
  double muw0;//reference dynamic viscosity of water
  double conb;
  double conc;
  double cpw;//water specific heat at const. pressure = 4181 J/kg.K
  double lambdaw;//water heat conductivity = 0.6 W/(m.K)
  double kw0;//compresibility coefficient of water

  //PHYSICAL PROPERTIES OF SOIL
  double alpha0; //initial Boit's constant
  double ks0;    //inital bulk modulus of solid phase
  double kt0;    //inital bulk modulus of porous medium
  double phi0;   //inital porosity
  double kintr0; //intial intrinsic permeability
  double betas0; //inital cubic thermal dilatation coefficient
  double deff0;  //initial effective diffusivity
  double rhos0;  //inital volume density of soil skeleton
  double cps0;   //inital specific heat of soil skeleton
  double lambda0;//effective thermal conductivity of soil skeleton
  double lambda_dry;//effective thermal conductivity of dry soil
  double lambda_wet;//effective thermal conductivity of saturated soil
  double sirr0; //irreversible (residual) saturation degree
  double tau0; //tortuosity factor 0.4 to 0.6

  // properties of van Genuchten model
  double ssat; //saturation = 1.0
  double expn; //exponent ussually = 2.0
  double delta; //parameter = 0.2 1/cm
  double gamaw; //water volume heaviness


  double mefel_units; //basic units for pressures = Pa (Pascals)

  double scale_pw,scale_pg,scale_t;
};  

#endif
